@@ -25,9 +25,9 @@ void prof_ver1to10(void)
 	QRcode *code;
 	int i;
 	int version;
-	static char *data = "This is test.";
+	static const char *data = "This is test.";
 
-	timerStart("Version 1 - 10");
+	timerStart("Version 1 - 10 (500 symbols for each)");
 	for(i=0; i<500; i++) {
 		for(version = 0; version < 11; version++) {
 			code = QRcode_encodeString(data, version, QR_ECLEVEL_L, QR_MODE_8, 0);
@@ -37,9 +37,27 @@ void prof_ver1to10(void)
 	timerStop();
 }
 
+void prof_ver31to40(void)
+{
+	QRcode *code;
+	int i;
+	int version;
+	static const char *data = "This is test.";
+
+	timerStart("Version 31 - 40 (50 symbols for each)");
+	for(i=0; i<50; i++) {
+		for(version = 31; version < 41; version++) {
+			code = QRcode_encodeString(data, version, QR_ECLEVEL_L, QR_MODE_8, 0);
+			QRcode_free(code);
+		}
+	}
+	timerStop();
+}
+
 int main()
 {
 	prof_ver1to10();
+	prof_ver31to40();
 
 	return 0;
 }
