@@ -9,7 +9,7 @@ void test_rscode1(void)
 {
 	QRinput *stream;
 	QRRawCode *code;
-	static char str[8] = "01234567";
+	static const char str[9] = "01234567";
 	static unsigned char correct[26] = {
 		0x10, 0x20, 0x0c, 0x56, 0x61, 0x80, 0xec, 0x11, 0xec, 0x11, 0xec, 0x11,
 		0xec, 0x11, 0xec, 0x11, 0xa5, 0x24, 0xd4, 0xc1, 0xed, 0x36, 0xc7, 0x87,
